import logging
from statsd.client import Timer


logger = logging.getLogger('django')


class LocalStatsClient(object):

    def timer(self, stat, rate=1):
        return Timer(self, stat, rate)

    def timing(self, stat, delta, rate=1):
        logger.info("STATSD_STAT::TIMING {} delta={} rate={}".format(stat, delta, rate))

    def incr(self, stat, count=1, rate=1):
        logger.info("STATSD_STAT::INCR {} count={} rate={}".format(stat, count, rate))

    def decr(self, stat, count=1, rate=1):
        logger.info("STATSD_STAT::DECR {} count={} rate={}".format(stat, count, rate))

    def gauge(self, stat, value, rate=1, delta=False):
        logger.info("STATSD_STAT::GAUGE {} value={} rate={} delta={}".format(stat, value, rate, delta))

    def set(self, stat, value, rate=1):
        logger.info("STATSD_STAT::SET {} value={} rate={}".format(stat, value, rate))



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from statsd.defaults.django import statsd as remote_statsd

from .statsd_client import LocalStatsClient


statsd = remote_statsd

STATSD_ENABLED = getattr(settings, 'STATSD_ENABLED', False)

if not STATSD_ENABLED:
    statsd = LocalStatsClient()



from django.test.runner import DiscoverRunner


class DatabaselessTestRunner(DiscoverRunner):
    """A test suite runner that does not set up and tear down a database."""

    def setup_databases(self):
        """Overrides DjangoTestSuiteRunner"""
        pass

    def teardown_databases(self, *args):
        """Overrides DjangoTestSuiteRunner"""
        pass



"""
A helper to alleviate some of the weirdness with django.utils.timezone.
This file also provides other additional methods to be used when
dealing with datetime.

To use exactly as django.utils.timezone do the following:
from zc_common import timezone
timezone.now()
timezone.get_current_timezone()
"""
import datetime as python_datetime


def now(tz=None):
    """
    Just like django.utils.timezone.now(), except:
    Takes a timezone as a param and defaults to non-utc
    """
    import pytz
    from django.conf import settings

    if settings.USE_TZ:
        tz = _get_tz(tz)
        now_dt = python_datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
        return localtime(now_dt, tz=tz)
    else:
        return python_datetime.datetime.now()


def get_next_weekday(weekday, tz=None):
    tnow = now(tz)
    weekdays = {
        'monday': 0,
        'tuesday': 1,
        'wednesday': 2,
        'thursday': 3,
        'friday': 4,
        'saturday': 5,
        'sunday': 6
    }
    weekday_num = weekdays[weekday.lower()]
    days_ahead = weekday_num - tnow.weekday()
    if days_ahead <= 0:  # Target day already happened this week
        days_ahead += 7
    return tnow + python_datetime.timedelta(days_ahead)


def localtime(value, tz=None):
    """
    Converts an aware datetime.datetime to a given time zone, defaults to currently activated time zone

    This method is taken almost directly from a later version of django.

    WARNING: In some cases (like with time math) normalize can cause times to shift by an hour
    when correcting the offset.
    """

    tz = _get_tz(tz)
    # If `value` is naive, astimezone() will raise a ValueError,
    # so we don't need to perform a redundant check.
    value = value.astimezone(tz)
    if hasattr(tz, 'normalize'):
        # This method is available for pytz time zones.
        value = tz.normalize(value)
    return value


def activate(value):
    """
    Override of django.utils.timezone.activate, but will first check
    if the value has a get_timezone method. If it does, the this method
    will use the value returned by get_timezone
    """
    import pytz

    if hasattr(value, 'get_timezone'):
        value = value.get_timezone()
    if isinstance(value, str):
        value = pytz.timezone(value)
    assert isinstance(value, python_datetime.tzinfo), 'Value passed was not tzinfo, it was: %s' % type(value)
    from django.utils.timezone import activate as timezone_activate
    return timezone_activate(value)


def is_daylight_savings_time(value):
    """
    Determines if the value is in daylight savings time.
    Can either take an aware datetime or a timezone object
    """
    new_datetime = _get_datetime_from_ambiguous_value(value)
    return new_datetime.dst() != python_datetime.timedelta(0)


def get_timezone_name(value):
    """
    Returns the current timezone name (PDT/PST).
    """
    return _get_datetime_from_ambiguous_value(value).strftime('%Z')


def get_timezone_offset(value):
    """
    Returns the current timezone offset (-0800).
    """
    return _get_datetime_from_ambiguous_value(value).strftime('%z')


def timezone_abbrv_mappings():
    """
    By default, dateutil doesn't parse at least `EDT` correctly.
    Pass output of this function as `tzinfos` param to parse() if it isn't pickin up timezone correctly.
    """
    from dateutil.tz import gettz
    return {'EDT': gettz('America/New_York'),
            'EST': gettz('America/New_York'),
            'CDT': gettz('America/Chicago'),
            'CST': gettz('America/Chicago'),
            'MDT': gettz('America/Denver'),
            'MST': gettz('America/Denver'),
            'PDT': gettz('America/Los_Angeles'),
            'PST': gettz('America/Los_Angeles')}


def _get_datetime_from_ambiguous_value(value):
    if type(value) is python_datetime.datetime:
        new_datetime = localtime(value, tz=value.tzinfo)
    elif type(value) is python_datetime.tzinfo:
        new_datetime = now(tz=value)
    else:
        raise Exception('value was not a timezone or a date, it was: %s' % type(value))
    return new_datetime


def combine(date, time, tz=None):
    """
    Like datetime.datetime.combine, but make it aware.
    Prefers timzeone that is passed in, followed by time.tzinfo, and then get_current_timezone
    """
    from django.utils.timezone import is_aware, make_aware

    if tz is None:
        tz = time.tzinfo
    tz = _get_tz(tz)
    combined = python_datetime.datetime.combine(date, time)
    return combined if is_aware(combined) else make_aware(combined, tz)


def parse(date_string, **kwargs):
    """ A wrapper around python-dateutil's parse function which ensures it always returns an aware datetime """
    from dateutil.parser import parse as datetime_parser
    from django.utils.timezone import is_aware, make_aware

    parsed = datetime_parser(date_string, **kwargs)
    # Make aware
    parsed = parsed if is_aware(parsed) else make_aware(parsed, _get_tz())
    # Ensure that we have the correct offset, while also keeping what was passed in.
    original = parsed
    parsed = localtime(parsed, tz=parsed.tzinfo).replace(
        year=original.year,
        month=original.month,
        day=original.day,
        hour=original.hour
    )
    return parsed


def datetime(year, month, day, hour=0, minute=0, second=0, microsecond=0, tzinfo=None):
    """
    A wrapper around datetime.datetime(), but ensures that the returned datetime is always
    timezone aware.
    """
    from django.utils.timezone import is_naive, make_aware

    tzinfo = _get_tz(tzinfo)
    dt = python_datetime.datetime(year, month, day, hour, minute, second, microsecond, tzinfo)
    if is_naive(dt):
        dt = make_aware(dt, tzinfo)
    dt = localtime(dt, tz=tzinfo)  # Have to set the correct offset
    # Setting the offset may have changed something else, like the hour, so replace
    return dt.replace(
        year=year, month=month, day=day, hour=hour, minute=minute, second=second, microsecond=microsecond)


def datetime_min():
    """ Returns datetime.datetime.min, but timezone aware """
    from django.utils.timezone import get_default_timezone
    return python_datetime.datetime.min.replace(tzinfo=get_default_timezone())


def datetime_max():
    """ Returns datetime.datetime.max, but timezone aware """
    from django.utils.timezone import get_default_timezone
    return python_datetime.datetime.max.replace(tzinfo=get_default_timezone())


def math(date, op, delta, keep_hour=False):
    """
    Performs operator math on datetime and timezone objects.
    This is needed when crossing daylight savings time thresholds to maintain
    the correct offset.

    WARNING FOR NON-UTC DATETIMES:
    If the daylight savings time threshold is crossed, the hour could change from under you.
    If this is not desired behaviour, pass in keep_hour=True.

    For example, if you have 7/1/2014 at midnight and you add 180 days to it, and keep_hour=False,
    it will return 12/27/2014 at 11 p.m. -- NOT 12/28/2014 at midnight like you might would expect.

    This is caused by pytz.normalize method.
    """
    converted = op(date, delta)
    original = converted
    converted = localtime(converted, tz=converted.tzinfo)  # Need to localize to get the timezone offset correct
    if keep_hour:
        if is_daylight_savings_time(date) != is_daylight_savings_time(converted):
            # Crossed the DST threshold
            # The hour doesn't change if datetime +/- timedelta
            # But does change when crossing DST and localizing
            converted = converted.replace(
                year=original.year,
                month=original.month,
                day=original.day,
                hour=original.hour
            )
    return converted


def javascript_iso_format(date):
    import pytz
    date = localtime(date, tz=pytz.utc)
    return date.strftime('%Y-%m-%dT%H:%M:%S') + 'Z'


def monthly_iter(start, end):
    """
    Iterates on a monthly basis, and wraps around dateutil.rrule

    Example:
    In [1]: from zc_common import timezone

    In [2]: for m in timezone.monthly_iter(timezone.datetime(2014, 10, 1), timezone.now()):
    ...:     print m
    ...:
    2014-10-01 00:00:00-07:00
    2014-11-01 00:00:00-07:00

    In [3]: for m in timezone.monthly_iter(timezone.datetime(2014, 10, 19), timezone.now()):
    ...:     print m
    ...:
    2014-10-19 00:00:00-07:00

    In [4]: timezone.now()
    Out[4]: datetime.datetime(
        2014, 11, 18, 16, 36, 54, 994666, tzinfo=<DstTzInfo 'America/Los_Angeles' PST-1 day, 16:00:00 STD>)
    """
    from dateutil import rrule

    for date in rrule.rrule(rrule.MONTHLY, dtstart=start, until=end):
        yield date


def weekly_iter(start, end, day=False):
    """
    Iterates weekly, wrapper around rrule.

    In [2]: for w in timezone.weekly_iter(timezone.datetime(2014, 7, 1), timezone.datetime(2014, 7, 31)):
    ...:     print w
    2014-07-01 00:00:00-07:00
    2014-07-08 00:00:00-07:00
    2014-07-15 00:00:00-07:00
    2014-07-22 00:00:00-07:00
    2014-07-29 00:00:00-07:00
    """
    from dateutil import rrule

    if day:
        while start.isoweekday() != day:
            start = start + python_datetime.timedelta(days=1)
    for date in rrule.rrule(rrule.WEEKLY, dtstart=start, until=end):
        yield date


def to_start_of_month(time):
    original_month = time.month
    original_year = time.year
    time.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
    time = localtime(time, time.tzinfo)
    # If the time crossed DST, the month and/or year may have changed.
    return time.replace(year=original_year, month=original_month,
                        day=1, hour=0, minute=0, second=0, microsecond=0)


def to_end_of_month(time):
    original_month = time.month
    original_year = time.year
    last_day = get_last_day_of_month(time)
    time.replace(day=last_day, hour=23, minute=59, second=59, microsecond=999999)
    time = localtime(time, time.tzinfo)
    # If the time crossed DST, the month and/or year may have changed.
    return time.replace(year=original_year, month=original_month,
                        day=last_day, hour=23, minute=59, second=59, microsecond=999999)


def to_start_of_day(time):
    return time.replace(hour=0, minute=0, second=0, microsecond=0)


def to_end_of_day(time):
    return time.replace(hour=23, minute=59, second=59, microsecond=999999)


def to_start_of_week(time):
    day = time.isoweekday()
    if day > 0:
        start = time - python_datetime.timedelta(days=day - 1)
    else:
        start = time
    return start


def to_end_of_week(time):
    day = time.isoweekday()
    if day > 0:
        start = time + python_datetime.timedelta(days=7 - day)
    else:
        start = time
    return start


def get_last_day_of_month(time):
    import calendar

    return calendar.monthrange(time.year, time.month)[1]


def is_business_day(time, include_weekends=True):
    """
    Determines if the current date is not a holiday.
    By default this includes weekends.
    """
    from django.conf import settings

    return not ((include_weekends and time.date().weekday() in [5, 6]) or  # saturday, sunday
                time.date() in settings.ZEROCATER_HOLIDAYS)


def _get_tz(tz=None):
    # Always get the current timezone, unless something is passed in
    from django.utils.timezone import get_current_timezone
    return tz if tz else get_current_timezone()


# http://aboutsimon.com/2013/06/05/datetime-hell-time-zone-aware-to-unix-timestamp/
def convert_to_timestamp(dt):
    import calendar
    import pytz
    from django.utils.timezone import is_aware
    if is_aware(dt):
        if dt.tzinfo != pytz.utc:
            dt = dt.astimezone(pytz.utc)
        return calendar.timegm(dt.timetuple())
    else:
        raise Exception('Can only convert aware datetimes to timestamps')


def convert_from_timestamp(timestamp):
    import pytz

    return python_datetime.datetime.utcfromtimestamp(timestamp).replace(tzinfo=pytz.utc)



from django.db.models import OneToOneField
from django.db.models.fields import related
from rest_framework.relations import ManyRelatedField
from rest_framework.utils.field_mapping import ClassLookupDict
from rest_framework_json_api.metadata import JSONAPIMetadata
from rest_framework_json_api.relations import ResourceRelatedField
from rest_framework_json_api.utils import get_related_resource_type

from zc_common.remote_resource.relations import RemoteResourceField
from zc_common.remote_resource.models import GenericRemoteForeignKey, RemoteForeignKey


class RelationshipMetadata(JSONAPIMetadata):
    relation_type_lookup = ClassLookupDict({
        related.ManyToManyDescriptor: 'ManyToMany',
        related.ReverseManyToOneDescriptor: 'OneToMany',
        related.ForwardManyToOneDescriptor: 'ManyToOne',
        related.ReverseOneToOneDescriptor: 'OneToOne',
        OneToOneField: 'OneToOne',
        RemoteForeignKey: 'ManyToOne',
        GenericRemoteForeignKey: 'ManyToOne'
    })

    def get_field_info(self, field):
        field_info = super(RelationshipMetadata, self).get_field_info(field)
        if isinstance(field, (RemoteResourceField, ManyRelatedField, ResourceRelatedField)):
            model_class = field.parent.Meta.model
            model_field = getattr(model_class, field.source)

            if hasattr(model_field, 'field') and isinstance(model_field.field, OneToOneField):
                # ForwardManyToOneDescriptor is used for OneToOneField also, so we have to override
                model_field = model_field.field

            field_info['relationship_type'] = self.relation_type_lookup[model_field]
            field_info['relationship_resource'] = get_related_resource_type(field)

            if field_info['relationship_resource'] == 'RemoteResource':
                field_info['relationship_resource'] = model_field.type
        return field_info



from __future__ import unicode_literals

from django.db import models
from django.db.models import signals


class RemoteResource(object):

    def __init__(self, type_name, pk):

        self.type = str(type_name) if type_name else None
        self.id = str(pk) if pk else None


class RemoteForeignKey(models.CharField):
    is_relation = True
    many_to_many = False
    many_to_one = True
    one_to_many = False
    one_to_one = False
    related_model = None
    remote_field = None

    description = "A foreign key pointing to an external resource"

    def __init__(self, type_name, *args, **kwargs):
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 50

        if 'db_index' not in kwargs:
            kwargs['db_index'] = True

        if 'db_column' not in kwargs:
            kwargs['db_column'] = "%s_id" % type_name.lower()

        self.type = type_name

        super(RemoteForeignKey, self).__init__(*args, **kwargs)

    def from_db_value(self, value, expression, connection, context):
        return RemoteResource(self.type, value)

    def to_python(self, value):
        if isinstance(value, RemoteResource):
            return value.id

        if isinstance(value, basestring):
            return value

        if value is None:
            return value

        raise ValueError("Can not convert value to a RemoteResource properly")

    def deconstruct(self):
        name, path, args, kwargs = super(RemoteForeignKey, self).deconstruct()

        args = tuple([self.type] + list(args))

        del kwargs['max_length']

        return name, path, args, kwargs

    def contribute_to_class(self, cls, name, **kwargs):
        self.set_attributes_from_name(name)
        self.name = name
        self.model = cls
        cls._meta.add_field(self)

        setattr(cls, name, self)


class GenericRemoteForeignKey(object):
    """
    Provide a generic many-to-one relation through the ``resource_type`` and
    ``resource_id`` fields.

    This class also doubles as an accessor to the related object (similar to
    Django ForeignKeys) by adding itself as a model attribute.
    """

    # Field flags
    auto_created = False
    concrete = False
    editable = False
    hidden = False

    is_relation = True
    many_to_many = False
    many_to_one = True
    one_to_many = False
    one_to_one = False
    related_model = None
    remote_field = None

    def __init__(self, resource_types=None, rt_field='resource_type', id_field='resource_id'):
        if resource_types is None:
            raise TypeError('resource_types cannot be None')
        self.resource_types = resource_types
        self.type = resource_types  # For metadata class
        self.rt_field = rt_field
        self.id_field = id_field
        self.editable = False
        self.rel = None
        self.column = None

    def contribute_to_class(self, cls, name, **kwargs):
        self.name = name
        self.model = cls
        self.cache_attr = "_%s_cache" % name
        cls._meta.add_field(self, virtual=True)

        # Only run pre-initialization field assignment on non-abstract models
        if not cls._meta.abstract:
            signals.pre_init.connect(self.instance_pre_init, sender=cls)

        setattr(cls, name, self)

    def is_cached(self, instance):
        return hasattr(instance, self.cache_attr)

    def instance_pre_init(self, signal, sender, args, kwargs, **_kwargs):
        """
        Handle initializing an object with the generic FK instead of
        content_type and object_id fields.
        """
        if self.name in kwargs:
            value = kwargs.pop(self.name)
            if value is not None:
                if not isinstance(value, RemoteResource):
                    raise ValueError(
                        'GenericRemoteForeignKey only accepts RemoteResource objects as values'
                    )
                kwargs[self.rt_field] = value.type
                kwargs[self.id_field] = value.id
            else:
                kwargs[self.rt_field] = None
                kwargs[self.id_field] = None

    def __get__(self, instance, instance_type=None):
        if instance is None:
            return self

        try:
            return getattr(instance, self.cache_attr)
        except AttributeError:
            resource_type = getattr(instance, self.rt_field)
            resource_id = getattr(instance, self.id_field)
            rel_obj = RemoteResource(resource_type, resource_id)
            setattr(instance, self.cache_attr, rel_obj)
            return rel_obj

    def __set__(self, instance, value):
        rt = None
        pk = None
        if value is not None:
            if not isinstance(value, RemoteResource):
                raise ValueError(
                    'GenericRemoteForeignKey only accepts RemoteResource objects as values'
                )
            if value.type not in self.resource_types:
                raise ValueError(
                    'Value must be of type {}, got {}'.format(self.resource_types, value.type)
                )
            rt = value.type
            pk = value.id

        setattr(instance, self.rt_field, rt)
        setattr(instance, self.id_field, pk)
        setattr(instance, self.cache_attr, value)



"""
Pagination fields

hamedahmadi 05/02/2016:
Had to create this custom pagination to include self link

see:
https://github.com/django-json-api/django-rest-framework-json-api/blob/develop/rest_framework_json_api/pagination.py
"""
from collections import OrderedDict

from django.utils.six.moves.urllib import parse as urlparse
from rest_framework.pagination import PageNumberPagination as OldPagination
from rest_framework.views import Response


def remove_query_param(url, key):
    """
    Given a URL and a key/val pair, remove an item in the query
    parameters of the URL, and return the new URL.

    Forked from rest_framework.utils.urls; overwriten here because we
    need to pass keep_blank_values=True to urlparse.parse_qs() so that
    it doesn't remove the ?ifilter[id__in]= blank query parameter from
    our links in the case of an empty remote to-many link.
    """
    (scheme, netloc, path, query, fragment) = urlparse.urlsplit(url)
    query_dict = urlparse.parse_qs(query, keep_blank_values=True)
    query_dict.pop(key, None)
    query = urlparse.urlencode(sorted(list(query_dict.items())), doseq=True)
    return urlparse.urlunsplit((scheme, netloc, path, query, fragment))


def replace_query_param(url, key, val):
    """
    Given a URL and a key/val pair, set or replace an item in the query
    parameters of the URL, and return the new URL.

    Forked from rest_framework.utils.urls; overwriten here because we
    need to pass keep_blank_values=True to urlparse.parse_qs() so that
    it doesn't remove the ?filter[id__in]= blank query parameter from
    our links in the case of an empty remote to-many link.
    """
    (scheme, netloc, path, query, fragment) = urlparse.urlsplit(url)
    query_dict = urlparse.parse_qs(query, keep_blank_values=True)
    query_dict[key] = [val]
    query = urlparse.urlencode(sorted(list(query_dict.items())), doseq=True)
    return urlparse.urlunsplit((scheme, netloc, path, query, fragment))


class PageNumberPagination(OldPagination):
    """
    A json-api compatible pagination format
    """

    page_size_query_param = 'page_size'
    max_page_size = 100

    def build_link(self, index):
        if not index:
            return None
        url = self.request and self.request.build_absolute_uri() or ''
        return replace_query_param(url, 'page', index)

    def get_paginated_response(self, data):
        next_page = None
        previous_page = None

        if self.page.has_next():
            next_page = self.page.next_page_number()
        if self.page.has_previous():
            previous_page = self.page.previous_page_number()

        # hamedahmadi 05/02/2016 -- Adding this to include self link
        self_url = remove_query_param(self.request.build_absolute_uri(), self.page_query_param)
        return Response({
            'results': data,
            'meta': {
                'pagination': OrderedDict([
                    ('page', self.page.number),
                    ('pages', self.page.paginator.num_pages),
                    ('count', self.page.paginator.count),
                ])
            },
            'links': OrderedDict([
                ('self', self_url),
                ('first', self.build_link(1)),
                ('last', self.build_link(self.page.paginator.num_pages)),
                ('next', self.build_link(next_page)),
                ('prev', self.build_link(previous_page))
            ])
        })



from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers
from rest_framework_json_api.utils import (
    get_resource_type_from_model, get_resource_type_from_instance)

from zc_common.remote_resource.models import RemoteResource


class ResourceIdentifierObjectSerializer(serializers.BaseSerializer):
    default_error_messages = {
        'incorrect_model_type': _('Incorrect model type. Expected {model_type}, received {received_type}.'),
        'does_not_exist': _('Invalid pk "{pk_value}" - object does not exist.'),
        'incorrect_type': _('Incorrect type. Expected pk value, received {data_type}.'),
    }

    model_class = None

    def __init__(self, *args, **kwargs):
        self.model_class = kwargs.pop('model_class', self.model_class)
        if 'instance' not in kwargs and not self.model_class:
            raise RuntimeError('ResourceIdentifierObjectsSerializer must be initialized with a model class.')
        super(ResourceIdentifierObjectSerializer, self).__init__(*args, **kwargs)

    def to_representation(self, instance):
        if isinstance(instance, RemoteResource):
            return {'type': instance.type, 'id': instance.id}

        return {
            'type': get_resource_type_from_instance(instance),
            'id': str(instance.pk)
        }

    def to_internal_value(self, data):
        model_class = get_resource_type_from_model(self.model_class)

        if model_class == "RemoteResource":
            return RemoteResource(data['type'], data['id'])

        if data['type'] != model_class:
            self.fail('incorrect_model_type', model_type=self.model_class, received_type=data['type'])
        pk = data['id']
        try:
            return self.model_class.objects.get(pk=pk)
        except ObjectDoesNotExist:
            self.fail('does_not_exist', pk_value=pk)
        except (TypeError, ValueError):
            self.fail('incorrect_type', data_type=type(data['pk']).__name__)



from rest_framework import permissions

USER_ACTOR = 'user'
STAFF_ACTOR = 'staff'
SERVICE_ACTOR = 'service'
ANONYMOUS_ACTOR = 'anonymous'

USER_ROLES = [USER_ACTOR]
STAFF_ROLES = [USER_ACTOR, STAFF_ACTOR]
SERVICE_ROLES = [SERVICE_ACTOR]
ANONYMOUS_ROLES = [ANONYMOUS_ACTOR]


def is_staff(request):
    return USER_ACTOR in request.user.roles and STAFF_ACTOR in request.user.roles


def is_user(request):
    return USER_ACTOR in request.user.roles


def is_service(request):
    return SERVICE_ACTOR in request.user.roles


def is_anonymous(request):
    return ANONYMOUS_ACTOR in request.user.roles


class BasePermission(permissions.BasePermission):
    def has_permission(self, request, view):
        if request.method in permissions.SAFE_METHODS:
            return self.has_read_permission(request, view)

        if request.method == 'DELETE':
            return self.has_delete_permission(request, view)

        if request.method in ['POST', 'PUT', 'PATCH']:
            return self.has_write_permission(request, view)

        return False

    def has_read_permission(self, request, view):
        return False

    def has_delete_permission(self, request, view):
        return False

    def has_write_permission(self, request, view):
        if request.method == 'POST':
            return self.has_create_permission(request, view)

        if request.method == 'PUT' or request.method == 'PATCH':
            return self.has_update_permission(request, view)

    def has_create_permission(self, request, view):
        return False

    def has_update_permission(self, request, view):
        return False


class EventViewPermission(BasePermission):
    def has_create_permission(self, request, view):
        return is_service(request)


class IsStaffPermission(permissions.BasePermission):
    def has_permission(self, request, view):
        return is_staff(request)



from __future__ import unicode_literals

import re
import uuid

from django.db import models
from django.core.validators import RegexValidator


US_PHONE_FORMAT = r'^(\+1\s?)?\(?(\d{3})\)?[\s-]?(\d{3})[\s-]?(\d{4})$'


def numeric_uuid_generator():
    return str(uuid.uuid4().int)[:10]


class PhoneNumberField(models.CharField):
    description = "A field that checks the value be a valid phone number"

    phone_regex = RegexValidator(
        regex=US_PHONE_FORMAT, message="Enter a valid Phone Number.")

    def __init__(self, *args, **kwargs):
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 20

        if 'validators' not in kwargs:
            kwargs['validators'] = [self.phone_regex]

        super(PhoneNumberField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if value is None:
            return value

        r = re.match(US_PHONE_FORMAT, value)
        if not r:
            return value

        return "({}) {}-{}".format(r.group(2), r.group(3), r.group(4))


class PKField(models.CharField):
    description = "A primary key field that defaults to a 10 digit random int"

    def __init__(self, *args, **kwargs):
        kwargs['primary_key'] = True

        if 'default' not in kwargs:
            kwargs['default'] = numeric_uuid_generator

        if 'max_length' not in kwargs:
            kwargs['max_length'] = 50

        super(PKField, self).__init__(*args, **kwargs)



"""
Parsers
"""
from rest_framework import parsers
from rest_framework.exceptions import ParseError

from rest_framework_json_api import utils, renderers, exceptions


class JSONParser(parsers.JSONParser):
    """
    A JSON API client will send a payload that looks like this:

        {
            "data": {
                "type": "identities",
                "id": 1,
                "attributes": {
                    "first_name": "John",
                    "last_name": "Coltrane"
                }
            }
        }

    We extract the attributes so that DRF serializers can work as normal.
    """
    media_type = 'application/vnd.api+json'
    renderer_class = renderers.JSONRenderer

    @staticmethod
    def parse_attributes(data):
        return utils.format_keys(data.get('attributes'), 'underscore') if data.get('attributes') else dict()

    @staticmethod
    def parse_relationships(data):
        relationships = (utils.format_keys(data.get('relationships'), 'underscore')
                         if data.get('relationships') else dict())

        # Parse the relationships
        parsed_relationships = dict()
        for field_name, field_data in relationships.items():
            field_data = field_data.get('data')
            if isinstance(field_data, dict) or field_data is None:
                parsed_relationships[field_name] = field_data
            elif isinstance(field_data, list):
                parsed_relationships[field_name] = list(relation for relation in field_data)
        return parsed_relationships

    @staticmethod
    def parse_metadata(result):
        metadata = result.get('meta')
        if metadata:
            return {'_meta': metadata}
        else:
            return {}

    def parse(self, stream, media_type=None, parser_context=None):
        """
        Parses the incoming bytestream as JSON and returns the resulting data
        """
        result = super(JSONParser, self).parse(stream, media_type=media_type, parser_context=parser_context)

        data = result.get('data')

        if data:
            from rest_framework_json_api.views import RelationshipView
            if isinstance(parser_context['view'], RelationshipView):
                # We skip parsing the object as JSONAPI Resource Identifier Object and not a regular Resource Object
                if isinstance(data, list):
                    for resource_identifier_object in data:
                        if not (resource_identifier_object.get('id') and resource_identifier_object.get('type')):
                            raise ParseError(
                                'Received data contains one or more malformed JSONAPI Resource Identifier Object(s)'
                            )
                elif not (data.get('id') and data.get('type')):
                    raise ParseError('Received data is not a valid JSONAPI Resource Identifier Object')

                return data

            request = parser_context.get('request')

            # Check for inconsistencies
            resource_name = utils.get_resource_name(parser_context)
            view = parser_context.get('view')
            if data.get('type') != resource_name and request.method in ('PUT', 'POST', 'PATCH'):
                raise exceptions.Conflict(
                    "The resource object's type ({data_type}) is not the type "
                    "that constitute the collection represented by the endpoint ({resource_type}).".format(
                        data_type=data.get('type'),
                        resource_type=resource_name
                    )
                )

            # Construct the return data
            parsed_data = {'id': data.get('id')}
            parsed_data.update(self.parse_attributes(data))
            parsed_data.update(self.parse_relationships(data))
            parsed_data.update(self.parse_metadata(result))
            return parsed_data

        else:
            raise ParseError('Received document does not contain primary data')



import re
from distutils.util import strtobool

from django.db.models import BooleanField, FieldDoesNotExist
from django.db.models.fields.related import ManyToManyField
from rest_framework import filters


class JSONAPIFilterBackend(filters.DjangoFilterBackend):
    def filter_queryset(self, request, queryset, view):
        filter_class = self.get_filter_class(view, queryset)
        primary_key = queryset.model._meta.pk.name

        query_params = {}

        for param, value in request.query_params.iteritems():
            match = re.search(r'^filter\[(\w+)\]$', param)
            if match:
                field_name = match.group(1)
                try:
                    name, extra = field_name.split('__')
                except ValueError:
                    name = field_name
                    extra = None
                if name not in view.filter_fields.keys():
                    return queryset.none()
                if len(field_name) > 1 and field_name[:2] == 'id':
                    query_params['{0}__{1}'.format(primary_key, extra)] = value
                if hasattr(queryset.model, field_name)\
                        and isinstance(getattr(queryset.model, field_name).field, ManyToManyField):
                    value = value.split(',')

                # Allow 'true' or 'false' as values for boolean fields
                try:
                    if isinstance(queryset.model._meta.get_field(field_name), BooleanField):
                        value = bool(strtobool(value))
                except FieldDoesNotExist:
                    pass

                query_params[field_name] = value

        if filter_class:
            return filter_class(query_params, queryset=queryset).qs

        return queryset



import os

from setuptools import setup


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


setup(
    name='zc_common',
    version='0.3.12b',
    description="Shared code for ZeroCater microservices",
    long_description='',
    keywords='zerocater python util',
    author='ZeroCater',
    author_email='tech@zerocater.com',
    url='https://github.com/ZeroCater/zc_common',
    download_url='https://github.com/ZeroCater/zc_common/tarball/0.3.12b',
    license='MIT',
    packages=get_packages('zc_common'),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP",
    ]
)
